@@ -43,6 +43,8 @@ static GstRegistry *_user_registry;
 static gboolean _gst_registry_fixed = FALSE;
 static gboolean _gst_use_threads = TRUE;
 
+static gboolean _gst_enable_cpu_opt = TRUE;
+
 static gboolean gst_initialized = FALSE;
 /* this will be set in popt callbacks when a problem has been encountered */
 static gboolean _gst_initialization_failure = FALSE;
@@ -82,6 +84,7 @@ enum {
   ARG_DEBUG_MASK,
   ARG_MASK,
   ARG_MASK_HELP,
+  ARG_DISABLE_CPU_OPT,
   ARG_PLUGIN_SPEW,
   ARG_PLUGIN_PATH,
   ARG_PLUGIN_LOAD,
@@ -94,7 +97,7 @@ enum {
 #define NUL '\0'
 #endif
 
-/* default scheduler, 'basicomega', can be changed in
+/* default scheduler, 'opt', can be changed in
  * gstscheduler.c in function gst_scheduler_factory_class_init
  */
 static const struct poptOption options[] = {
@@ -105,10 +108,11 @@ static const struct poptOption options[] = {
   {"gst-debug-mask",     NUL, POPT_ARG_INT|POPT_ARGFLAG_STRIP,    NULL, ARG_DEBUG_MASK,     "debugging bitmask", "MASK"},
   {"gst-mask",           NUL, POPT_ARG_INT|POPT_ARGFLAG_STRIP,    NULL, ARG_MASK,           "bitmask for both info and debugging", "MASK"},
   {"gst-mask-help",      NUL, POPT_ARG_NONE|POPT_ARGFLAG_STRIP,   NULL, ARG_MASK_HELP,      "how to set the level of diagnostic output (-mask values)", NULL},
+  {"gst-disable-cpu-opt",NUL, POPT_ARG_NONE|POPT_ARGFLAG_STRIP,	  NULL, ARG_DISABLE_CPU_OPT,"Disable accelerated CPU instructions", NULL},
   {"gst-plugin-spew",    NUL, POPT_ARG_NONE|POPT_ARGFLAG_STRIP,   NULL, ARG_PLUGIN_SPEW,    "enable verbose plugin loading diagnostics", NULL},
   {"gst-plugin-path",    NUL, POPT_ARG_STRING|POPT_ARGFLAG_STRIP, NULL, ARG_PLUGIN_PATH,    "'" G_SEARCHPATH_SEPARATOR_S "'--separated path list for loading plugins", "PATHS"},
   {"gst-plugin-load",    NUL, POPT_ARG_STRING|POPT_ARGFLAG_STRIP, NULL, ARG_PLUGIN_LOAD,    "comma-separated list of plugins to preload in addition to the list stored in env variable GST_PLUGIN_PATH", "PLUGINS"},
-  {"gst-scheduler",      NUL, POPT_ARG_STRING|POPT_ARGFLAG_STRIP, NULL, ARG_SCHEDULER,      "scheduler to use ('basicomega' is the default)", "SCHEDULER"},
+  {"gst-scheduler",      NUL, POPT_ARG_STRING|POPT_ARGFLAG_STRIP, NULL, ARG_SCHEDULER,      "scheduler to use ('opt' is the default)", "SCHEDULER"},
   {"gst-nothreads",      NUL, POPT_ARG_NONE|POPT_ARGFLAG_STRIP,   NULL, ARG_NOTHREADS,      "use NOPs for all threading and locking operations", NULL},
   {"gst-registry",       NUL, POPT_ARG_STRING|POPT_ARGFLAG_STRIP, NULL, ARG_REGISTRY,       "registry to use" , "REGISTRY"},
   POPT_TABLEEND
@@ -166,8 +170,10 @@ gst_init_check (int *argc, char **argv[])
 void
 gst_init (int *argc, char **argv[])
 {
-  if (!gst_init_with_popt_table (argc, argv, NULL))
-    g_error ("Could not initialize GStreamer !\n");
+  if (!gst_init_with_popt_table (argc, argv, NULL)) {
+    g_print ("Could not initialize GStreamer !\n");
+    exit (1);
+  }
 }
 
 /**
@@ -190,19 +196,59 @@ gst_init_with_popt_table (int *argc, char **argv[],
   gint nextopt, i, j, nstrip;
   gchar **temp;
   const struct poptOption *options;
-  /* this is probably hacky, no? */
-  const struct poptOption options_with[] = {
-    {NULL, NUL, POPT_ARG_INCLUDE_TABLE, poptHelpOptions, 				 0, "Help options:", NULL},
-    {NULL, NUL, POPT_ARG_INCLUDE_TABLE, (struct poptOption *) gst_init_get_popt_table(), 0, "GStreamer options:", NULL},
-    {NULL, NUL, POPT_ARG_INCLUDE_TABLE, (struct poptOption *) popt_options, 		 0, "Application options:", NULL},
-    POPT_TABLEEND
+  struct poptOption options_with[] = {
+     POPT_TABLEEND,
+     POPT_TABLEEND,
+     POPT_TABLEEND,
+     POPT_TABLEEND
   };
-  const struct poptOption options_without[] = {
-    {NULL, NUL, POPT_ARG_INCLUDE_TABLE, poptHelpOptions, 				 0, "Help options:", NULL},
-    {NULL, NUL, POPT_ARG_INCLUDE_TABLE, (struct poptOption *) gst_init_get_popt_table(), 0, "GStreamer options:", NULL},
-    POPT_TABLEEND
+  struct poptOption options_without[] = {
+     POPT_TABLEEND,
+     POPT_TABLEEND,
+     POPT_TABLEEND
   };
 
+  /* This used to be done by struct initialization, but most
+   * compilers don't like calling functions in a struct
+   * initialization.  (It's a GCC extension.)  FIXME: This should
+   * be reworked to look better. */
+  options_with[0].longName   = NULL;
+  options_with[0].shortName  = NUL;
+  options_with[0].argInfo    = POPT_ARG_INCLUDE_TABLE;
+  options_with[0].arg        = poptHelpOptions;
+  options_with[0].val        = 0;
+  options_with[0].descrip    = "Help options:";
+  options_with[0].argDescrip = NULL;
+  options_with[1].longName   = NULL;
+  options_with[1].shortName  = NUL;
+  options_with[1].argInfo    = POPT_ARG_INCLUDE_TABLE;
+  options_with[1].arg        = (struct poptOption *) gst_init_get_popt_table();
+  options_with[1].val        = 0;
+  options_with[1].descrip    = "GStreamer options:";
+  options_with[1].argDescrip = NULL;
+  options_with[2].longName   = NULL;
+  options_with[2].shortName  = NUL;
+  options_with[2].argInfo    = POPT_ARG_INCLUDE_TABLE;
+  options_with[2].arg        = (struct poptOption *) popt_options;
+  options_with[2].val        = 0;
+  options_with[2].descrip    = "Application options:";
+  options_with[2].argDescrip = NULL;
+
+  options_without[0].longName   = NULL;
+  options_without[0].shortName  = NUL;
+  options_without[0].argInfo    = POPT_ARG_INCLUDE_TABLE;
+  options_without[0].arg        = poptHelpOptions;
+  options_without[0].val        = 0;
+  options_without[0].descrip    = "Help options:";
+  options_without[0].argDescrip = NULL;
+  options_without[1].longName   = NULL;
+  options_without[1].shortName  = NUL;
+  options_without[1].argInfo    = POPT_ARG_INCLUDE_TABLE;
+  options_without[1].arg        = (struct poptOption *) gst_init_get_popt_table();
+  options_without[1].val        = 0;
+  options_without[1].descrip    = "GStreamer options:";
+  options_without[1].argDescrip = NULL;
+
   if (gst_initialized)
   {
     GST_DEBUG (GST_CAT_GST_INIT, "already initialized gst\n");
@@ -352,26 +398,17 @@ init_pre (void)
   gst_registry_add_path (_global_registry, PLUGINS_DIR);
 #endif /* PLUGINS_USE_BUILDDIR */
 
-  homedir = g_get_home_dir ();
-  user_reg = g_strjoin ("/", homedir, LOCAL_REGISTRY_FILE, NULL);
+  if (g_getenv ("GST_REGISTRY"))
+  {
+    user_reg = g_strdup (g_getenv ("GST_REGISTRY"));
+  }
+  else
+  {
+    homedir = g_get_home_dir ();
+    user_reg = g_strjoin ("/", homedir, LOCAL_REGISTRY_FILE, NULL);
+  }
   _user_registry = gst_xml_registry_new ("user_registry", user_reg);
 
-#ifndef GST_DISABLE_REGISTRY
-  /* this test is a hack; gst-register sets this to false
-   * so this is a test for the current instance being gst-register */
-  if (_gst_registry_auto_load == TRUE)
-  {
-    /* do a sanity check here; either one of the two registries should exist */
-    if (!g_file_test (user_reg, G_FILE_TEST_IS_REGULAR))
-      if (!g_file_test (GLOBAL_REGISTRY_FILE, G_FILE_TEST_IS_REGULAR))
-      {
-        g_print ("Couldn't find user registry %s or global registry %s\n",
-	         user_reg, GLOBAL_REGISTRY_FILE);
-        g_error ("Please run gst-register either as root or user");
-      }
-  }
-#endif
-	  
   g_free (user_reg);
 
   return TRUE;
@@ -455,6 +492,7 @@ init_post (void)
   gst_autoplug_factory_get_type ();
 #endif
   gst_index_factory_get_type ();
+  gst_uri_handler_get_type ();
 
 
   plugin_path = g_getenv ("GST_PLUGIN_PATH");
@@ -463,7 +501,7 @@ init_post (void)
   /* register core plugins */
   _gst_plugin_register_static (&plugin_desc);
  
-  _gst_cpu_initialize ();
+  _gst_cpu_initialize (_gst_enable_cpu_opt);
   _gst_props_initialize ();
   _gst_caps_initialize ();
   _gst_plugin_initialize ();
@@ -576,6 +614,9 @@ init_popt_callback (poptContext context, enum poptCallbackReason reason,
     case ARG_MASK_HELP:
       gst_mask_help ();
       exit (0);
+    case ARG_DISABLE_CPU_OPT:
+      _gst_enable_cpu_opt = FALSE;
+      break;
     case ARG_PLUGIN_SPEW:
       break;
     case ARG_PLUGIN_PATH:
