@@ -146,6 +146,10 @@ void		gst_object_restore_thyself	(GstObject *object, xmlNodePtr self);
 GstObject *	gst_object_ref			(GstObject *object);
 void 		gst_object_unref		(GstObject *object);
 void 		gst_object_sink			(GstObject *object);
+
+/* replace object pointer */
+void 		gst_object_swap			(GstObject **oldobj, GstObject *newobj);
+
 /* destroying an object */
 void 		gst_object_destroy		(GstObject *object);
 
