@@ -46,9 +46,11 @@ GType _gst_pad_type = 0;
 /***** Start with the base GstPad class *****/
 static void		gst_pad_class_init		(GstPadClass *klass);
 static void		gst_pad_init			(GstPad *pad);
+static void		gst_pad_dispose			(GObject *object);
 
 static gboolean 	gst_pad_try_relink_filtered_func (GstRealPad *srcpad, GstRealPad *sinkpad, 
 							 GstCaps *caps, gboolean clear);
+static void		gst_pad_set_pad_template	(GstPad *pad, GstPadTemplate *templ);
 
 #ifndef GST_DISABLE_LOADSAVE
 static xmlNodePtr	gst_pad_save_thyself		(GstObject *object, xmlNodePtr parent);
@@ -76,15 +78,28 @@ gst_pad_get_type (void)
 static void
 gst_pad_class_init (GstPadClass *klass)
 {
+  GObjectClass *gobject_class;
+
+  gobject_class = (GObjectClass*) klass;
+
   pad_parent_class = g_type_class_ref (GST_TYPE_OBJECT);
+
+  gobject_class->dispose = GST_DEBUG_FUNCPTR (gst_pad_dispose);
 }
 
 static void
 gst_pad_init (GstPad *pad)
 {
-  pad->element_private = NULL;
+  /* all structs are initialized to NULL by glib */
+}
+static void
+gst_pad_dispose (GObject *object)
+{
+  GstPad *pad = GST_PAD (object);
 
-  pad->padtemplate = NULL;
+  gst_pad_set_pad_template (pad, NULL);
+
+  G_OBJECT_CLASS (pad_parent_class)->dispose (object);
 }
 
 
@@ -108,6 +123,7 @@ enum {
 
 static void	gst_real_pad_class_init		(GstRealPadClass *klass);
 static void	gst_real_pad_init		(GstRealPad *pad);
+static void	gst_real_pad_dispose		(GObject *object);
 
 static void	gst_real_pad_set_property	(GObject *object, guint prop_id,
                                                  const GValue *value, 
@@ -116,8 +132,6 @@ static void	gst_real_pad_get_property	(GObject *object, guint prop_id,
                                                  GValue *value, 
 						 GParamSpec *pspec);
 
-static void	gst_real_pad_dispose		(GObject *object);
-
 GType _gst_real_pad_type = 0;
 
 static GstPad *real_pad_parent_class = NULL;
@@ -322,12 +336,8 @@ gst_pad_custom_new_from_template (GType type, GstPadTemplate *templ,
   g_return_val_if_fail (templ != NULL, NULL);
 
   pad = gst_pad_new (name, templ->direction);
-  
-  gst_object_ref (GST_OBJECT (templ));
-  GST_PAD_PAD_TEMPLATE (pad) = templ;
+  gst_pad_set_pad_template (pad, templ);
 
-  g_signal_emit (G_OBJECT (templ), gst_pad_template_signals[TEMPL_PAD_CREATED], 0, pad);
-  
   return pad;
 }
 
@@ -741,7 +751,7 @@ gst_pad_set_formats_function (GstPad *pad, GstPadFormatsFunction formats)
  */
 void
 gst_pad_set_link_function (GstPad *pad,
-		              GstPadLinkFunction link)
+	                   GstPadLinkFunction link)
 {
   g_return_if_fail (pad != NULL);
   g_return_if_fail (GST_IS_REAL_PAD (pad));
@@ -751,6 +761,25 @@ gst_pad_set_link_function (GstPad *pad,
              GST_DEBUG_PAD_NAME (pad), GST_DEBUG_FUNCPTR_NAME (link));
 }
 
+/**
+ * gst_pad_set_unlink_function:
+ * @pad: a #GstPad to set the unlink function for.
+ * @link: the #GstPadLinkFunction to set.
+ *
+ * Sets the given unlink function for the pad. It will be called
+ * when the pad is unlinked from its peer.
+ */
+void
+gst_pad_set_unlink_function (GstPad *pad, GstPadUnlinkFunction link)
+{
+  g_return_if_fail (pad != NULL);
+  g_return_if_fail (GST_IS_REAL_PAD (pad));
+
+  GST_RPAD_UNLINKFUNC (pad) = link;
+  GST_DEBUG (GST_CAT_PADS, "unlinkfunc for %s:%s set to %s",
+             GST_DEBUG_PAD_NAME (pad), GST_DEBUG_FUNCPTR_NAME (link));
+}
+
 /**
  * gst_pad_set_getcaps_function:
  * @pad: a #GstPad to set the getcaps function for.
@@ -842,9 +871,8 @@ gst_pad_unlink (GstPad *srcpad,
 
   /* reset the filters, both filters are refcounted once */
   if (GST_RPAD_FILTER (realsrc)) {
-    gst_caps_unref (GST_RPAD_FILTER (realsrc));
-    GST_RPAD_FILTER (realsink) = NULL;
-    GST_RPAD_FILTER (realsrc) = NULL;
+    gst_caps_replace (&GST_RPAD_FILTER (realsink), NULL);
+    gst_caps_replace (&GST_RPAD_FILTER (realsrc), NULL);
   }
 
   /* now tell the scheduler */
@@ -962,7 +990,7 @@ gst_pad_can_link (GstPad *srcpad, GstPad *sinkpad)
  * @filtercaps: the filter #GstCaps.
  *
  * Links the source pad and the sink pad, constrained
- * by the given filter caps.
+ * by the given filter caps. This function sinks the caps.
  *
  * Returns: TRUE if the pads have been linked, FALSE otherwise.
  */
@@ -1065,11 +1093,14 @@ gst_pad_link_filtered (GstPad *srcpad, GstPad *sinkpad, GstCaps *filtercaps)
     gst_scheduler_pad_link (src_sched, 
 	                       GST_PAD_CAST (realsrc), GST_PAD_CAST (realsink));
   }
+  else {
+    GST_INFO (GST_CAT_PADS, "not telling link to scheduler %s:%s and %s:%s, %p %p",
+            GST_DEBUG_PAD_NAME (srcpad), GST_DEBUG_PAD_NAME (sinkpad),
+            src_sched, sink_sched);
+  }
 
   GST_INFO (GST_CAT_PADS, "linked %s:%s and %s:%s, successful",
             GST_DEBUG_PAD_NAME (srcpad), GST_DEBUG_PAD_NAME (sinkpad));
-  gst_caps_debug (gst_pad_get_caps (GST_PAD_CAST (realsrc)), 
-                  "caps of newly linked src pad");
 
   return TRUE;
 }
@@ -1126,6 +1157,16 @@ gst_pad_get_parent (GstPad *pad)
   return GST_PAD_PARENT (pad);
 }
 
+static void
+gst_pad_set_pad_template (GstPad *pad, GstPadTemplate *templ)
+{
+  /* this function would need checks if it weren't static */
+
+  gst_object_swap ((GstObject **) &pad->padtemplate, (GstObject *) templ);
+  
+  if (templ)
+    g_signal_emit (G_OBJECT (templ), gst_pad_template_signals[TEMPL_PAD_CREATED], 0, pad);
+} 
 /**
  * gst_pad_get_pad_template:
  * @pad: a #GstPad to get the pad template of.
@@ -1235,14 +1276,13 @@ gst_pad_remove_ghost_pad (GstPad *pad,
 {
   GstRealPad *realpad;
 
-  g_return_if_fail (pad != NULL);
   g_return_if_fail (GST_IS_PAD (pad));
-  g_return_if_fail (ghostpad != NULL);
   g_return_if_fail (GST_IS_GHOST_PAD (ghostpad));
-
   realpad = GST_PAD_REALIZE (pad);
+  g_return_if_fail (GST_GPAD_REALPAD (ghostpad) == realpad);
 
   realpad->ghostpads = g_list_remove (realpad->ghostpads, ghostpad);
+  GST_GPAD_REALPAD (ghostpad) = NULL;
 }
 
 /**
@@ -1271,7 +1311,7 @@ gst_pad_get_ghost_pad_list (GstPad *pad)
 static GstPadLinkReturn
 gst_pad_try_set_caps_func (GstRealPad *pad, GstCaps *caps, gboolean notify)
 {
-  GstCaps *oldcaps, *allowed = NULL;
+  GstCaps *allowed = NULL;
   GstPadTemplate *template;
   GstElement *parent = GST_PAD_PARENT (pad);
 
@@ -1291,7 +1331,7 @@ gst_pad_try_set_caps_func (GstRealPad *pad, GstCaps *caps, gboolean notify)
             caps, GST_DEBUG_PAD_NAME (pad));
 
   /* first see if we have to check against a filter */
-  if (!(allowed = GST_RPAD_FILTER (pad))) {
+  if (!(allowed = gst_caps_ref (GST_RPAD_FILTER (pad)))) {
     /* no filter, make sure we check against the padtemplate then */
     if ((template = gst_pad_get_pad_template (GST_PAD_CAST (pad)))) {
       allowed = gst_pad_template_get_caps (template);
@@ -1312,10 +1352,12 @@ gst_pad_try_set_caps_func (GstRealPad *pad, GstCaps *caps, gboolean notify)
       gst_caps_debug (caps, "caps themselves (attemped to set)");
       gst_caps_debug (allowed,
                       "allowed caps that did not agree with caps");
+      gst_caps_unref (allowed);
       return GST_PAD_LINK_REFUSED;
     }
     /* caps checks out fine, we can unref the intersection now */
     gst_caps_unref (intersection);
+    gst_caps_unref (allowed);
     /* given that the caps are fixed, we know that their intersection with the
      * padtemplate caps is the same as caps itself */
   }
@@ -1383,10 +1425,7 @@ gst_pad_try_set_caps_func (GstRealPad *pad, GstCaps *caps, gboolean notify)
     GST_INFO (GST_CAT_CAPS, "setting caps on pad %s:%s",
               GST_DEBUG_PAD_NAME (pad));
     /* if we got this far all is ok, remove the old caps, set the new one */
-    oldcaps = GST_PAD_CAPS (pad);
-    if (caps) gst_caps_ref (caps);
-    GST_PAD_CAPS (pad) = caps;
-    if (oldcaps) gst_caps_unref (oldcaps);
+    gst_caps_replace_sink (&GST_PAD_CAPS (pad), caps);
 
     g_object_notify (G_OBJECT (pad), "caps");
   }
@@ -1403,7 +1442,8 @@ gst_pad_try_set_caps_func (GstRealPad *pad, GstCaps *caps, gboolean notify)
  * @pad: a #GstPad to try to set the caps on.
  * @caps: the #GstCaps to set.
  *
- * Tries to set the caps on the given pad.
+ * Tries to set the caps on the given pad.  Takes ownership of the caps;
+ * if you want to reuse these caps you need to ref them first.
  *
  * Returns: A #GstPadLinkReturn value indicating whether the caps
  * 		could be set.
@@ -1422,6 +1462,10 @@ gst_pad_try_set_caps (GstPad *pad, GstCaps *caps)
 
   gst_caps_debug (caps, "caps that we are trying to set");
 
+  /* try to take ownership */
+  gst_caps_ref (caps);
+  gst_caps_sink (caps);
+
   /* setting non fixed caps on a pad is not allowed */
   if (!GST_CAPS_IS_FIXED (caps)) {
   GST_INFO (GST_CAT_CAPS, 
@@ -1430,7 +1474,8 @@ gst_pad_try_set_caps (GstPad *pad, GstCaps *caps)
     g_warning ("trying to set non fixed caps on pad %s:%s, not allowed",
                GST_DEBUG_PAD_NAME (realpad));
     gst_caps_debug (caps, "unfixed caps");
-    return GST_PAD_LINK_DELAYED;
+    set_retval = GST_PAD_LINK_DELAYED;
+    goto done;
   }
 
   /* if we have a peer try to set the caps, notifying the peerpad
@@ -1439,7 +1484,7 @@ gst_pad_try_set_caps (GstPad *pad, GstCaps *caps)
   {
     GST_INFO (GST_CAT_CAPS, "tried to set caps on peerpad %s:%s but couldn't, return value %d",
 	      GST_DEBUG_PAD_NAME (peer), set_retval);
-    return set_retval;
+    goto done;
   }
 
   /* then try to set our own caps, we don't need to be notified */
@@ -1447,11 +1492,14 @@ gst_pad_try_set_caps (GstPad *pad, GstCaps *caps)
   {
     GST_INFO (GST_CAT_CAPS, "tried to set own caps on pad %s:%s but couldn't, return value %d",
 	      GST_DEBUG_PAD_NAME (realpad), set_retval);
-    return set_retval;
+    goto done;
   }
   GST_INFO (GST_CAT_CAPS, "succeeded setting caps %p on pad %s:%s, return value %d",
 	    caps, GST_DEBUG_PAD_NAME (realpad), set_retval);
   g_assert (GST_PAD_CAPS (pad));
+
+done:
+  gst_caps_unref (caps);
 			  
   return set_retval;
 }
@@ -1485,8 +1533,10 @@ gst_pad_try_relink_filtered_func (GstRealPad *srcpad, GstRealPad *sinkpad,
 	      "start relink filtered %s:%s and %s:%s, clearing caps",
               GST_DEBUG_PAD_NAME (realsrc), GST_DEBUG_PAD_NAME (realsink));
 
-    GST_PAD_CAPS (GST_PAD (realsrc)) = NULL;
-    GST_PAD_CAPS (GST_PAD (realsink)) = NULL;
+    gst_caps_replace (&GST_PAD_CAPS (GST_PAD (realsrc)), NULL);
+    gst_caps_replace (&GST_PAD_CAPS (GST_PAD (realsink)), NULL);
+    gst_caps_replace (&GST_RPAD_FILTER (realsrc), NULL);
+    gst_caps_replace (&GST_RPAD_FILTER (realsink), NULL);
   }
   else {
     GST_INFO (GST_CAT_PADS, "start relink filtered %s:%s and %s:%s",
@@ -1512,6 +1562,9 @@ gst_pad_try_relink_filtered_func (GstRealPad *srcpad, GstRealPad *sinkpad,
      * this means they have no common format */
     GST_INFO (GST_CAT_PADS, "pads %s:%s and %s:%s have no common type",
               GST_DEBUG_PAD_NAME (realsrc), GST_DEBUG_PAD_NAME (realsink));
+    /* make sure any floating caps from gst_pad_get_caps are freed here */
+    gst_caps_sink (srccaps);
+    gst_caps_sink (sinkcaps);
     return FALSE;
   } else  {
     GST_INFO (GST_CAT_PADS, "pads %s:%s and %s:%s intersected to %s caps",
@@ -1520,13 +1573,19 @@ gst_pad_try_relink_filtered_func (GstRealPad *srcpad, GstRealPad *sinkpad,
 	   (GST_CAPS_IS_FIXED (intersection) ? "fixed" : "variable") :
 	   "NULL"));
 
+    /* we don't need those anymore, as the caps can be floating */
+    gst_caps_sink (srccaps);
+    gst_caps_sink (sinkcaps);
+
     /* then filter this against the app filter */
     if (filtercaps) {
-      GstCaps *filtered_intersection = gst_caps_intersect (intersection, 
-	                                                   filtercaps);
+      GstCaps *filtered_intersection;
+	      
+      filtered_intersection = gst_caps_intersect (intersection, 
+	                                          filtercaps);
 
       /* get rid of the old intersection here */
-      gst_caps_unref (intersection);
+      gst_caps_sink (intersection);
 
       if (!filtered_intersection) {
         GST_INFO (GST_CAT_PADS, 
@@ -1537,8 +1596,9 @@ gst_pad_try_relink_filtered_func (GstRealPad *srcpad, GstRealPad *sinkpad,
       intersection = filtered_intersection;
 
       /* keep a reference to the app caps */
-      GST_RPAD_APPFILTER (realsink) = filtercaps;
-      GST_RPAD_APPFILTER (realsrc) = filtercaps;
+      gst_caps_replace_sink (&GST_RPAD_APPFILTER (realsink), filtercaps);
+      gst_caps_replace_sink (&GST_RPAD_APPFILTER (realsrc), filtercaps);
+      gst_caps_sink (intersection);
     }
   }
   GST_DEBUG (GST_CAT_CAPS, "setting filter for link to:");
@@ -1546,8 +1606,9 @@ gst_pad_try_relink_filtered_func (GstRealPad *srcpad, GstRealPad *sinkpad,
 
   /* both the app filter and the filter, while stored on both peer pads, 
    * are equal to the same thing on both */
-  GST_RPAD_FILTER (realsrc) = intersection; 
-  GST_RPAD_FILTER (realsink) = intersection; 
+  gst_caps_replace_sink (&GST_RPAD_FILTER (realsrc), intersection);
+  gst_caps_replace_sink (&GST_RPAD_FILTER (realsink), intersection);
+  gst_caps_unref (intersection);
 
   return gst_pad_perform_negotiate (GST_PAD (realsrc), GST_PAD (realsink));
 }
@@ -1567,6 +1628,8 @@ gst_pad_perform_negotiate (GstPad *srcpad, GstPad *sinkpad)
   GstCaps *intersection, *filtered_intersection;
   GstRealPad *realsrc, *realsink;
   GstCaps *srccaps, *sinkcaps, *filter;
+  gboolean res = TRUE;
+  GstElement *parent;
 
   g_return_val_if_fail (srcpad != NULL, FALSE);
   g_return_val_if_fail (sinkpad != NULL, FALSE);
@@ -1577,6 +1640,24 @@ gst_pad_perform_negotiate (GstPad *srcpad, GstPad *sinkpad)
   g_return_val_if_fail (GST_RPAD_PEER (realsrc) != NULL, FALSE);
   g_return_val_if_fail (GST_RPAD_PEER (realsink) == realsrc, FALSE);
 
+  /* shortcut negotiation */
+  parent = GST_PAD_PARENT (realsrc);
+  if (parent && GST_STATE (parent) < GST_STATE_READY) {
+    GST_DEBUG (GST_CAT_CAPS, "parent %s of pad %s:%s is not READY",
+               GST_ELEMENT_NAME (parent), GST_DEBUG_PAD_NAME (realsrc));
+    return TRUE;
+  }
+  parent = GST_PAD_PARENT (realsink);
+  if (parent && GST_STATE (parent) < GST_STATE_READY) {
+    GST_DEBUG (GST_CAT_CAPS, "parent %s of pad %s:%s is not READY",
+               GST_ELEMENT_NAME (parent), GST_DEBUG_PAD_NAME (realsink));
+    return TRUE;
+  }
+
+  GST_INFO (GST_CAT_PADS, "perform negotiate for link %s:%s-%s:%s",
+              GST_DEBUG_PAD_NAME (realsrc), GST_DEBUG_PAD_NAME (realsink));
+
+
   filter = GST_RPAD_APPFILTER (realsrc);
   if (filter) {
     GST_INFO (GST_CAT_PADS, "dumping filter for link %s:%s-%s:%s",
@@ -1597,28 +1678,39 @@ gst_pad_perform_negotiate (GstPad *srcpad, GstPad *sinkpad)
                   "sink caps, awaiting negotiation, after applying filter");
   intersection = gst_caps_intersect (srccaps, sinkcaps);
   filtered_intersection = gst_caps_intersect (intersection, filter);
-  if (filtered_intersection) {
-    gst_caps_unref (intersection);
-    intersection = filtered_intersection;
-  }
+  gst_caps_unref (intersection);
 
   /* no negotiation is performed if the pads have filtercaps */
-  if (intersection) {
-    GstPadLinkReturn res;
+  if (filtered_intersection) {
+    GstPadLinkReturn link_res;
 
-    res = gst_pad_try_set_caps_func (realsrc, intersection, TRUE);
-    if (res == GST_PAD_LINK_REFUSED) 
-      return FALSE;
-    if (res == GST_PAD_LINK_DONE) 
-      return TRUE;
+    link_res = gst_pad_try_set_caps_func (realsrc, filtered_intersection, TRUE);
+    if (link_res == GST_PAD_LINK_REFUSED)
+      goto error;
+    if (link_res == GST_PAD_LINK_DONE)
+      goto success;
 
-    res = gst_pad_try_set_caps_func (realsink, intersection, TRUE);
-    if (res == GST_PAD_LINK_REFUSED) 
-      return FALSE;
-    if (res == GST_PAD_LINK_DONE) 
-      return TRUE;
+    link_res = gst_pad_try_set_caps_func (realsink, filtered_intersection, TRUE);
+    if (link_res == GST_PAD_LINK_REFUSED)
+      goto error;
+    if (link_res == GST_PAD_LINK_DONE)
+      goto success;
   }
-  return TRUE;
+  /* no filtered_intersection, some pads had caps and ther was a filter */
+  else if ((srccaps || sinkcaps) && filter) {
+    goto error;
+  }
+
+success:
+cleanup:
+  gst_caps_sink (srccaps);
+  gst_caps_sink (sinkcaps);
+  gst_caps_unref (filtered_intersection);
+  return res;
+
+error:
+  res = FALSE;
+  goto cleanup;
 }
 
 /**
@@ -1712,6 +1804,16 @@ gst_pad_proxy_link (GstPad *pad, GstCaps *caps)
   if (gst_pad_try_set_caps_func (realpad, caps, FALSE) < 0)
     return GST_PAD_LINK_REFUSED;
 
+  if (peer) {
+    gst_caps_debug (caps, "proxy link filter");
+
+    GST_INFO (GST_CAT_CAPS, "setting filter on %s:%s and %s:%s",
+              GST_DEBUG_PAD_NAME (peer), GST_DEBUG_PAD_NAME (realpad));
+
+    gst_caps_replace_sink (&GST_RPAD_FILTER (peer), caps);
+    gst_caps_replace_sink (&GST_RPAD_FILTER (realpad), caps);
+  }
+
   return GST_PAD_LINK_OK;
 }
 
@@ -1721,7 +1823,9 @@ gst_pad_proxy_link (GstPad *pad, GstCaps *caps)
  *
  * Gets the capabilities of this pad.
  *
- * Returns: the #GstCaps of this pad.
+ * Returns: the #GstCaps of this pad. This function potentially
+ * returns a floating caps, so use gst_caps_sink to get rid of
+ * it. 
  */
 GstCaps*
 gst_pad_get_caps (GstPad *pad)
@@ -1761,7 +1865,8 @@ gst_pad_get_caps (GstPad *pad)
  *
  * Gets the template capabilities of this pad.
  *
- * Returns: the template #GstCaps of this pad.
+ * Returns: the template #GstCaps of this pad. unref the caps
+ * if you no longer need it.
  */
 GstCaps*
 gst_pad_get_pad_template_caps (GstPad *pad)
@@ -1770,7 +1875,7 @@ gst_pad_get_pad_template_caps (GstPad *pad)
   g_return_val_if_fail (GST_IS_PAD (pad), NULL);
 
   if (GST_PAD_PAD_TEMPLATE (pad))
-    return GST_PAD_TEMPLATE_CAPS (GST_PAD_PAD_TEMPLATE (pad));
+    return gst_caps_ref (GST_PAD_TEMPLATE_CAPS (GST_PAD_PAD_TEMPLATE (pad)));
 
   return NULL;
 }
@@ -1782,7 +1887,8 @@ gst_pad_get_pad_template_caps (GstPad *pad)
  *
  * Gets the capability with the given name from this pad template.
  *
- * Returns: the #GstCaps, or NULL if not found or in case of an error.
+ * Returns: the #GstCaps, or NULL if not found or in case of an error. unref the caps
+ * if you no longer need it.
  */
 GstCaps*
 gst_pad_template_get_caps_by_name (GstPadTemplate *templ, const gchar *name)
@@ -1795,7 +1901,7 @@ gst_pad_template_get_caps_by_name (GstPadTemplate *templ, const gchar *name)
   if (!caps) 
     return NULL;
 
-  return gst_caps_get_by_name (caps, name);
+  return gst_caps_ref (gst_caps_get_by_name (caps, name));
 }
 
 /**
@@ -1858,7 +1964,8 @@ gst_pad_get_peer (GstPad *pad)
  * Gets the capabilities of the allowed media types that can
  * flow through this pad.  The caller must free the resulting caps.
  *
- * Returns: a newly allocated copy of the allowed #GstCaps.
+ * Returns: a newly allocated copy of the allowed #GstCaps. unref the caps if
+ * you no longer need it.
  */
 GstCaps*
 gst_pad_get_allowed_caps (GstPad *pad)
@@ -1871,7 +1978,7 @@ gst_pad_get_allowed_caps (GstPad *pad)
   GST_DEBUG (GST_CAT_PROPERTIES, "get allowed caps of %s:%s", 
              GST_DEBUG_PAD_NAME (pad));
 
-  caps = gst_caps_copy (GST_RPAD_FILTER (pad));
+  caps = gst_caps_ref (GST_RPAD_FILTER (pad));
 
   return caps;
 }
@@ -1900,7 +2007,7 @@ gst_pad_recalc_allowed_caps (GstPad *pad)
   peer = GST_RPAD_PEER (pad);
   if (peer)
     return gst_pad_try_relink_filtered (pad, GST_PAD (peer), 
-	                                   GST_RPAD_APPFILTER (pad));
+	                                GST_RPAD_APPFILTER (pad));
 
   return TRUE;
 }
@@ -1955,13 +2062,6 @@ gst_real_pad_dispose (GObject *object)
 
   GST_DEBUG (GST_CAT_REFCOUNTING, "dispose %s:%s", GST_DEBUG_PAD_NAME(pad));
 
-  if (GST_PAD_PAD_TEMPLATE (pad)){
-    GST_DEBUG (GST_CAT_REFCOUNTING, "unreffing padtemplate'%s'", 
-	       GST_OBJECT_NAME (GST_PAD_PAD_TEMPLATE (pad)));
-    gst_object_unref (GST_OBJECT (GST_PAD_PAD_TEMPLATE (pad)));
-    GST_PAD_PAD_TEMPLATE (pad) = NULL;
-  }
-  
   /* we destroy the ghostpads, because they are nothing without the real pad */
   if (GST_REAL_PAD (pad)->ghostpads) {
     GList *orig, *ghostpads;
@@ -1983,6 +2083,9 @@ gst_real_pad_dispose (GObject *object)
     g_list_free (GST_REAL_PAD(pad)->ghostpads);
   }
 
+  gst_caps_replace (&GST_PAD_CAPS (pad), NULL);
+  gst_caps_replace (&GST_RPAD_APPFILTER (pad), NULL);
+
   if (GST_IS_ELEMENT (GST_OBJECT_PARENT (pad))) {
     GST_DEBUG (GST_CAT_REFCOUNTING, "removing pad from element '%s'",
                GST_OBJECT_NAME (GST_OBJECT (GST_ELEMENT (GST_OBJECT_PARENT (pad)))));
@@ -2151,8 +2254,9 @@ gst_pad_push (GstPad *pad, GstBuffer *buf)
 	       GST_DEBUG_PAD_NAME (pad));
   }
   else {
-    if (!GST_IS_EVENT (buf) && !GST_PAD_IS_ACTIVE (pad)) {
-      g_warning ("push on pad %s:%s but it is not active", 
+    if (!GST_IS_EVENT (buf) && !GST_PAD_IS_ACTIVE (peer)) {
+      g_warning ("push on peer %s:%s of pad %s:%s but peer is not active", 
+	         GST_DEBUG_PAD_NAME (peer),
 	         GST_DEBUG_PAD_NAME (pad));
       return;
     }
@@ -2310,6 +2414,7 @@ gst_pad_selectv (GstPad *pad, ...)
  */
 static void		gst_pad_template_class_init	(GstPadTemplateClass *klass);
 static void		gst_pad_template_init		(GstPadTemplate *templ);
+static void             gst_pad_template_dispose        (GObject *object);
 
 GType
 gst_pad_template_get_type (void)
@@ -2347,6 +2452,8 @@ gst_pad_template_class_init (GstPadTemplateClass *klass)
 		  NULL, NULL, gst_marshal_VOID__POINTER, G_TYPE_NONE, 1,
                   G_TYPE_POINTER);
 
+  gobject_class->dispose = gst_pad_template_dispose;
+
   gstobject_class->path_string_separator = "*";
 }
 
@@ -2355,6 +2462,17 @@ gst_pad_template_init (GstPadTemplate *templ)
 {
 }
 
+static void
+gst_pad_template_dispose (GObject *object)
+{
+  GstPadTemplate *templ = GST_PAD_TEMPLATE (object);
+
+  g_free (GST_PAD_TEMPLATE_NAME_TEMPLATE (templ));
+  gst_caps_unref (GST_PAD_TEMPLATE_CAPS (templ));
+
+  G_OBJECT_CLASS (padtemplate_parent_class)->dispose (object);
+}
+
 /* ALWAYS padtemplates cannot have conversion specifications, it doesn't make
  * sense.
  * SOMETIMES padtemplates can do whatever they want, they are provided by the
@@ -2433,14 +2551,19 @@ gst_pad_template_new (const gchar *name_template,
 
   va_start (var_args, caps);
 
+  GST_FLAG_SET (GST_OBJECT (new), GST_PAD_TEMPLATE_FIXED);
   while (caps) {
-    new->fixed &= caps->fixed;
-    thecaps = gst_caps_append (thecaps, gst_caps_ref (caps));
+    if (!GST_CAPS_IS_FIXED (caps)) {
+      GST_FLAG_UNSET (GST_OBJECT (new), GST_PAD_TEMPLATE_FIXED);
+    }
+    thecaps = gst_caps_append (thecaps, caps);
     caps = va_arg (var_args, GstCaps*);
   }
   va_end (var_args);
   
   GST_PAD_TEMPLATE_CAPS (new) = thecaps;
+  gst_caps_ref (thecaps);
+  gst_caps_sink (thecaps);
 
   return new;
 }
@@ -2458,7 +2581,7 @@ gst_pad_template_get_caps (GstPadTemplate *templ)
 {
   g_return_val_if_fail (templ != NULL, NULL);
 
-  return GST_PAD_TEMPLATE_CAPS (templ);
+  return gst_caps_ref (GST_PAD_TEMPLATE_CAPS (templ));
 }
 
 /**
@@ -2493,8 +2616,9 @@ gst_pad_get_element_private (GstPad *pad)
 /***** ghost pads *****/
 GType _gst_ghost_pad_type = 0;
 
-static void     gst_ghost_pad_class_init         (GstGhostPadClass *klass);
-static void     gst_ghost_pad_init               (GstGhostPad *pad);
+static void     gst_ghost_pad_class_init        (GstGhostPadClass *klass);
+static void     gst_ghost_pad_init              (GstGhostPad *pad);
+static void     gst_ghost_pad_dispose		(GObject *object);
 
 static GstPad *ghost_pad_parent_class = NULL;
 /* static guint gst_ghost_pad_signals[LAST_SIGNAL] = { 0 }; */
@@ -2525,12 +2649,24 @@ gst_ghost_pad_class_init (GstGhostPadClass *klass)
   gobject_class = (GObjectClass*) klass;
 
   ghost_pad_parent_class = g_type_class_ref (GST_TYPE_PAD);
+
+  gobject_class->dispose = GST_DEBUG_FUNCPTR (gst_ghost_pad_dispose);
 }
 
 static void
 gst_ghost_pad_init (GstGhostPad *pad)
 {
-  pad->realpad = NULL;
+  /* zeroed by glib */
+}
+static void
+gst_ghost_pad_dispose (GObject *object)
+{
+  GstGhostPad *pad = GST_GHOST_PAD (object);
+
+  if (pad->realpad)
+    gst_pad_remove_ghost_pad((GstPad *) pad->realpad, (GstPad *) pad);
+
+  G_OBJECT_CLASS (ghost_pad_parent_class)->dispose (object);
 }
 
 /**
@@ -2540,7 +2676,7 @@ gst_ghost_pad_init (GstGhostPad *pad)
  *
  * Creates a new ghost pad associated with the given pad, and names it with
  * the given name.  If name is NULL, a guaranteed unique name (across all
- * ghost pads) will be assigned (most likely of the form ghostpad&perc;d).
+ * ghost pads) will be assigned (most likely of the form ghostpad%d).
  *
  * Returns: a new ghost #GstPad, or NULL in case of an error.
  */
@@ -2563,7 +2699,7 @@ gst_ghost_pad_new (const gchar *name,
     realpad = GST_PAD_REALIZE (realpad);
   }
   GST_GPAD_REALPAD (ghostpad) = realpad;
-  GST_PAD_PAD_TEMPLATE (ghostpad) = GST_PAD_PAD_TEMPLATE (pad);
+  gst_pad_set_pad_template (GST_PAD (ghostpad), GST_PAD_PAD_TEMPLATE (pad));
 
   /* add ourselves to the real pad's list of ghostpads */
   gst_pad_add_ghost_pad (pad, GST_PAD (ghostpad));
@@ -2785,8 +2921,8 @@ gst_pad_send_event (GstPad *pad, GstEvent *event)
   GST_DEBUG (GST_CAT_EVENT, "have event %d on pad %s:%s",
 		  GST_EVENT_TYPE (event), GST_DEBUG_PAD_NAME (rpad));
 
-  if (GST_RPAD_EVENTFUNC (rpad))
-    success = GST_RPAD_EVENTFUNC (rpad) (GST_PAD_CAST (rpad), event);
+  if (GST_RPAD_EVENTHANDLER (rpad))
+    success = GST_RPAD_EVENTHANDLER (rpad) (GST_PAD_CAST (rpad), event);
   else {
     GST_DEBUG (GST_CAT_EVENT, "there's no event function for pad %s:%s", 
 	       GST_DEBUG_PAD_NAME (rpad));
