@@ -111,7 +111,7 @@ queue_leaky_get_type(void) {
 }
 
 static GstElementClass *parent_class = NULL;
-/* static guint gst_queue_signals[LAST_SIGNAL] = { 0 }; */
+static guint gst_queue_signals[LAST_SIGNAL] = { 0 };
 
 GType
 gst_queue_get_type(void) 
@@ -147,6 +147,12 @@ gst_queue_class_init (GstQueueClass *klass)
 
   parent_class = g_type_class_ref (GST_TYPE_ELEMENT);
 
+  gst_queue_signals[HIGH_WATERMARK] =
+    g_signal_new ("high_watermark", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_FIRST,
+                  G_STRUCT_OFFSET (GstQueueClass, high_watermark), NULL, NULL,
+                  g_cclosure_marshal_VOID__INT, G_TYPE_NONE, 1,
+                  G_TYPE_INT);
+
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_LEAKY,
     g_param_spec_enum ("leaky", "Leaky", "Where the queue leaks, if at all.",
                        GST_TYPE_QUEUE_LEAKY, GST_QUEUE_NO_LEAK, G_PARAM_READWRITE));
@@ -194,11 +200,14 @@ gst_queue_getcaps (GstPad *pad, GstCaps *caps)
   GstPad *otherpad;
 
   if (pad == queue->srcpad) 
-    otherpad = queue->sinkpad;
+    otherpad = GST_PAD_PEER (queue->sinkpad);
   else
-    otherpad = queue->srcpad;
+    otherpad = GST_PAD_PEER (queue->srcpad);
 
-  return gst_pad_get_allowed_caps (otherpad);
+  if (otherpad)
+    return gst_pad_get_caps (otherpad);
+
+  return NULL;
 }
 
 static void
@@ -250,6 +259,8 @@ gst_queue_dispose (GObject *object)
 {
   GstQueue *queue = GST_QUEUE (object);
 
+  gst_element_set_state (GST_ELEMENT (queue), GST_STATE_NULL);
+
   g_mutex_free (queue->qlock);
   g_cond_free (queue->not_empty);
   g_cond_free (queue->not_full);
@@ -345,6 +356,11 @@ restart:
   GST_DEBUG_ELEMENT (GST_CAT_DATAFLOW, queue, "adding buffer %p of size %d",buf,GST_BUFFER_SIZE(buf));
 
   if (queue->level_buffers == queue->size_buffers) {
+    g_mutex_unlock (queue->qlock);
+    g_signal_emit (G_OBJECT (queue), gst_queue_signals[HIGH_WATERMARK], 0,
+                   queue->level_buffers);
+    g_mutex_lock (queue->qlock);
+
     /* if this is a leaky queue... */
     if (queue->leaky) {
       /* FIXME don't want to leak events! */
@@ -671,7 +687,7 @@ gst_queue_change_state (GstElement *element)
           GST_DEBUG_ELEMENT (GST_CAT_STATES, queue, "queue %s does not connect different schedulers", 
 			GST_ELEMENT_NAME (queue));
 
-	  g_warning ("queue %s does connect different schedulers",
+	  g_warning ("queue %s does not connect different schedulers",
 			GST_ELEMENT_NAME (queue));
 
 	  ret = GST_STATE_FAILURE;
