@@ -26,8 +26,8 @@
 
 #define GST_VERSION_MAJOR 0
 #define GST_VERSION_MINOR 6
-#define GST_VERSION_MICRO 0
+#define GST_VERSION_MICRO 4
 
 void    gst_version     (guint *major, guint *minor, guint *micro);
 
-#endif /* __GST_H__ */
+#endif /* __GST_VERSION_H__ */
